--[[
一个简单的示例，新建一个精灵，并在精灵上放置了一个文本。
当调用 CreateCustomNode 时，会生成这个精灵并返回。
]]

-- 新建一个 table，包含 CreateCustomNode 和 GetBaseType 方法。
local container = {}

local Parser ={}
container.RichTextSys = {}
container.RichTextSys["DEFAULT_PARSER_LABEL"]={["FONT"]="font",["IMAGE"]="img"} --可以解析的标签
container.RichTextSys["DEFAULT"]={["rgba"]={255,255,255,255},["size"]=24,["fontName"]="Helvetica"} --默认字体框效果
container.RichTextSys["isAllLine"]=false --是否按行进行排版
container.RichTextSys["DEFAULT_HOLDERPLACE"]="　"--默认

-- 新建根节点 Node，目前这个方法的名字为固定的，必须为 CreateCustomNode。
-- 方法的最后一句必须是一个 return 语句，把新建的结点返回。
function container.CreateCustomNode()
	local rootNode = nil
    if __NotEditor then
    	rootNode = ccui.RichText:create()
    	rootNode.setTextHorizontalAlignment=function(self,horizontal) end
    	rootNode.setTextVerticalAlignment=function(self,vertical) end
		rootNode.setTextAreaSize=function(self,areaSize)end
		rootNode.setCustom=rootNode.ignoreContentAdaptWithSize
		rootNode.ignoreContentAdaptWithSize=function(self,isIgnore)
			self["isCustom"]= (not isIgnore)
			self:setCustom(isIgnore)
		end
		rootNode.setFontSize=function(self,fontSize)
			self["calcFontSize"]=fontSize
			container.RichTextSys:setDefault(self,fontSize,nil,nil)
			container.RichTextSys:reBuild(nil,self)
		end
		rootNode.setString=function(self,string)
			local str = lang.get(string)
			if str=="" then
				str=" "
			end
			container.RichTextSys:reBuild(str,self)
		end
		rootNode.setFontName=function(self,fontName)
			container.RichTextSys:setDefault(self,nil,fontName,nil)
			container.RichTextSys:reBuild(nil,self)
		end
		rootNode.setCascadeColorEnabled=function(self,isEnabled)	end
		rootNode.setCascadeOpacityEnabled=function(self,isEnabled)	end
		rootNode.setColor=function(self,color)
			container.RichTextSys:setDefault(self,nil,nil,{color.r,color.g,color.b})
			container.RichTextSys:reBuild(nil,self)
		end
		rootNode.setCustomWidth=function (self,customWidth)
			self["customWidth"]=customWidth
			container.RichTextSys:reBuild(nil,self)
		end
		rootNode.getCustomWidth=function (self)
			local result=nil
			if nil~=self["customWidth"] then
				result=self["customWidth"]
			end
			return result
		end
        rootNode.enableOutline=function(self, outlineColor, outlineSize)
            self._outline = true
            self._outlineColor = outlineColor
            self._outlineSize = outlineSize
            if self.parsingString then
                container.RichTextSys:reBuild(self.parsingString,self)
            end
        end
		rootNode.cloneUI=rootNode.clone
		rootNode.clone=function(self)
			local curCloneUI=ccui.RichText:create()
			curCloneUI.setTextHorizontalAlignment=self.setTextHorizontalAlignment
			curCloneUI.setTextVerticalAlignment=self.setTextVerticalAlignment
			curCloneUI.setTextAreaSize=self.setTextAreaSize
			curCloneUI.setCustom=self.setCustom
			curCloneUI.ignoreContentAdaptWithSize=self.ignoreContentAdaptWithSize
			curCloneUI.setFontSize=self.setFontSize
			curCloneUI.setFontName=self.setFontName
			curCloneUI.setString=self.setString
			curCloneUI.setCascadeColorEnabled=self.setCascadeColorEnabled
			curCloneUI.setCascadeOpacityEnabled=self.setCascadeOpacityEnabled
			curCloneUI.setColor=self.setColor
			curCloneUI.setCustomWidth=self.setCustomWidth
			curCloneUI.getCustomWidth=self.getCustomWidth
            curCloneUI.clone=self.clone
			curCloneUI:setName(self:getName().."_clone")
			curCloneUI:setPosition(cc.p(self:getPositionX()+10,self:getPositionY()+10))
			curCloneUI:setContentSize(self:getContentSize())
			curCloneUI:setAnchorPoint(self:getAnchorPoint())
			curCloneUI:setColor(cc.c3b(unpack(self.parsingDefault.rgba)))
			curCloneUI:setFontSize(self.parsingDefault.size)
			curCloneUI:setFontName(self.parsingDefault.fontName)
			curCloneUI:ignoreContentAdaptWithSize(not self["isCustom"])
			curCloneUI["customWidth"]=self["customWidth"]
            curCloneUI.enableOutline=self.enableOutline
            curCloneUI._outline=self._outline
            curCloneUI._outlineColor=self._outlineColor
            curCloneUI._outlineSize=self._outlineSize
			return curCloneUI
		end
    else
        rootNode = ccui.Text:create()
    end
	return rootNode
end

function container.RichTextSys:setDefault(node,fontSize,fontName,fontColor3b)
	if type(node)=="userdata" then
		if nil==node["parsingDefault"] then
			node["parsingDefault"]={["rgba"]=self["DEFAULT"].rgba,["size"]=self["DEFAULT"].size,["fontName"]=self["DEFAULT"].fontName}
		end
		local curDefault = node["parsingDefault"]
		if nil~=curDefault then
			if type(fontSize)=="number" then
				curDefault.size=fontSize
			end
			if type(fontName)=="string" then
				curDefault.fontName=fontName
			end
			if type(fontColor3b)=="table" and #fontColor3b-3==0 then
				curDefault.rgba={fontColor3b[1],fontColor3b[2],fontColor3b[3],255}
			end
			node["parsingDefault"]=curDefault
		end
	end
end

function container.RichTextSys:reBuild(str,node)
	if type(node)=="userdata" then
		if type(str)=="string" and str~="" then
			node["parsingString"]=str
		end
		if nil~=node["parsingString"] then
			if type(node["parsingCount"])=="number" and node["parsingCount"]-1>=0 then
				for j=1,node["parsingCount"] do
					node:removeElement(0)
				end
			end
			local curRichTextInfo = Parser:parse(node["parsingString"])
			local curParsingCount = #curRichTextInfo
			node["parsingCount"] = curParsingCount
			if nil==node["parsingDefault"] then
				node["parsingDefault"]={["rgba"]=self["DEFAULT"].rgba,["size"]=self["DEFAULT"].size,["fontName"]=self["DEFAULT"].fontName}
			end
			local curContentWidth=0
			local isNewLine=false
			local widthArr={}
			local elementArr={}
			if curParsingCount-1>=0 then
				for i=1,curParsingCount do
					local element = self:generateElement(node,curRichTextInfo[i],i)
					if nil~=element then
						if curRichTextInfo[i]["isNewLine"] and i~=1 then
							isNewLine=true
							table.insert(widthArr,{i,curContentWidth})
							curContentWidth=0
						end
						if nil~=element["ui"] then
							curContentWidth=curContentWidth+element["ui"]:getContentSize().width
						end
						node:pushBackElement(element)
						table.insert(elementArr,element)
					end
				end
				if isNewLine then
					table.insert(widthArr,{node["parsingCount"]+1,curContentWidth})
					local maxWidth=0
					for k,v in pairs(widthArr) do
						if v[2]-maxWidth>0 then
							maxWidth = v[2]
						end
					end
					local curWidthCount=#widthArr
					for n=1,curWidthCount do
						if n==1 then
							if widthArr[n][1]==2 then
								widthArr[n]["isBlock"]=true
								elementArr[1].ui:setContentSize(maxWidth,elementArr[1].ui:getContentSize().height)
							end
						else
							local curLine=widthArr[n][1]
							local preLine=widthArr[n-1][1]
							if curLine-preLine==1 then
								widthArr[n]["isBlock"]=true
								elementArr[preLine].ui:setContentSize(maxWidth,elementArr[preLine].ui:getContentSize().height)
							end
						end
					end
					local curInsertCount=0
					for j=1,curWidthCount-1 do
						local insertIndex=widthArr[j][1]-1+curInsertCount
						if nil==widthArr[j]["isBlock"] then
							if insertIndex>0 then
								node:insertElement(self:generateEmptyElement(node, insertIndex,maxWidth-widthArr[j][2],node["parsingDefault"].size),insertIndex)
								node["parsingCount"]=node["parsingCount"]+1
								curInsertCount=curInsertCount+1
							end
						end
					end
					node:ignoreContentAdaptWithSize(false)
					node:setContentSize(cc.size(maxWidth,node["parsingDefault"].size*curWidthCount*1.2))
				elseif node["isCustom"] then
					if nil~=node["customWidth"] and type(node["customWidth"])=="number" then
                        node:formatText()
                        node:setContentSize(cc.size(node["customWidth"], node:getContentSize().height))
--                        local curStrContentSize=self:calcStrContentSize(node,curRichTextInfo)
--						if nil~=curStrContentSize then
--							local lines=math.ceil(curStrContentSize.width/node["customWidth"])
--							node:setContentSize(cc.size(node["customWidth"],node["calcFontSize"]*lines*1.2))
--						end
					end
				end
			end
		end
	end
end

function container.RichTextSys:generateEmptyElement(node, index,width,fontSize)
	local result=nil
	if type(index)=="number" and type(width)=="number" then
		local txtUI=ccui.Text:create()
		local count=width/fontSize
		txtUI:setFontSize(fontSize)
		local str=""
		if count-1>0 then
			for i=1,count do
				str=str..self["DEFAULT_HOLDERPLACE"]
			end
		end
		txtUI:setString(str)
        if node._outline then
            txtUI:enableOutline(node._outlineColor, node._outlineSize)
        end
		result=ccui.RichElementCustomNode:create(index,cc.c3b(255,0,0), 255, txtUI)
	end
	return result
end

function container.RichTextSys:generateElement(node,elementInfo,elementIndex)
	local element=nil
	if type(node)=="userdata" and type(elementInfo)=="table" and type(elementIndex)=="number" then
		if nil~=elementInfo["isAllLine"] then
			self["isAllLine"]=true
		end
		local curText=""
		if nil~=elementInfo.text then
			curText=elementInfo.text
		end
		local curColor=cc.c3b(node["parsingDefault"].rgba[1],node["parsingDefault"].rgba[2],node["parsingDefault"].rgba[3])
		local curOpacity=node["parsingDefault"].rgba[4]
		local isCustomColor=false
		if nil~=elementInfo.color then
			isCustomColor=true
			curColor=cc.c3b(elementInfo.color["1"],elementInfo.color["2"],elementInfo.color["3"])
		end
		local curSize=node["parsingDefault"].size
		if nil~=elementInfo.size then
			curSize = elementInfo.size
		end
		local curImagePath=""
		if nil~=elementInfo.src then
			curImagePath=elementInfo.src
		end
		local curFontName = node["parsingDefault"].fontName
		if nil~=elementInfo.fontName then
			curFontName = elementInfo.fontName --得到的是字库的路径
		end
		if elementInfo.type == self["DEFAULT_PARSER_LABEL"].FONT then
			if nil~=elementInfo.isLine or self["isAllLine"] then
				local txtUI=ccui.Text:create()
				txtUI:setString(curText)
				txtUI:setFontSize(curSize)
				txtUI:setColor(curColor)
				txtUI:setFontName(curFontName)
				if node._outline then
                    txtUI:enableOutline(node._outlineColor, node._outlineSize)
                end
				txtUI:ignoreContentAdaptWithSize(false)
                if nil~=elementInfo.alignH then
					txtUI:setTextHorizontalAlignment(elementInfo.alignH+0)
				end
				if nil~=elementInfo.alignV then
					txtUI:setTextVerticalAlignment(elementInfo.alignV+0)
				end
				element=ccui.RichElementCustomNode:create(elementIndex,curColor, curOpacity, txtUI)
				element["ui"]=txtUI
			else
                if node._outline then
				    element=ccui.RichElementText:create(elementIndex, curColor, curOpacity, curText, curFontName, curSize, node._outlineColor, node._outlineSize )
                else
                    element=ccui.RichElementText:create(elementIndex, curColor, curOpacity, curText, curFontName, curSize)
                end
			end
		elseif  elementInfo.type == self["DEFAULT_PARSER_LABEL"].IMAGE then
			if cc.FileUtils:getInstance():isFileExist(curImagePath) then
				if not isCustomColor then
					curColor = cc.c3b(255,255,255)
				end
				element=ccui.RichElementImage:create(elementIndex,curColor, curOpacity, curImagePath)
			else
				element=ccui.RichElementCustomNode:create(elementIndex, cc.c3b(255, 255, 255), curOpacity, cc.Node:create())
			end
		end
	end
	return element
end

function container.RichTextSys:calcStrContentSize(node,info)
	local result=nil
	if type(node)=="userdata" and type(info)=="table" then
		local actualStr=""
		for i=1,#info do
			if info[i].type=="font" then
				actualStr=string.format("%s%s",actualStr,info[i].text)
			end
		end
		local calcText=ccui.Text:create()
		calcText:setString(actualStr)
		calcText:setFontSize(node["calcFontSize"])
		node["calcContentSize"]=calcText:getContentSize()
		result=node["calcContentSize"]
	end
	return result
end
-------------------------------------------
-------------------------------------------
-- local Parser ={}
	Parser["PARSE_LABEL"] = {["FONT"]="font",["IMAGE"]="img"} --可以解析的标签
	Parser["PARSE_ATTRIBUTE"] ={["COLOR"]="color",["SIZE"]="size",["FONT_NAME"]="fontName",["SRC"]="src",["IS_LINE"]="isLine",["IS_ALL_LINE"]="isAllLine",["IS_NEW_LINE"]="isNewLine",["ALIGN_H"]="alignH",["ALIGN_V"]="alignV"} --可以解析的属性
	Parser["PARSE_HOLDPLACE"] ="||MrC||" --替换占位符
	Parser["PARSE_INDEX"] ="%sIndex" --标签数据的起始位置索引
	Parser["PARSE_REMAIN"] = "remain" --解析后剩下的数据(标签)
	Parser["FORMAT_PREFIX_LABEL_PATTERN"] = "<%s " --前缀解析标签
	Parser["FORMAT_SUFFIX_LABEL_PATTERN"] = "</%s>" --后缀解析标签
	Parser["FORMAT_ATTRIBUTE_PATTERN"] = " ([^ ]+)='([^ ^>]+)'" --属性解析标签(依赖前缀、后缀标签首字符)
	Parser["FORMAT_INNER_TEXT_PATTERN"] = ">([^>]*)<" --文本解析标签(依赖前缀、后缀标签首/尾字符)

	local curSectionsArr = {[Parser["PARSE_REMAIN"]]=""} --语法分析标签后得到数据分组
	local curSectionsIndexArr ={} --语法分析标签后得到数据分组的索引
	local curParsedArr = {} --解析完成的数据
	local curCombineArr = {} --重组解析完成的数据

	--[[一些清理工作]]
	function Parser:clear()
		curSectionsArr = {[Parser["PARSE_REMAIN"]]=""}
		curSectionsIndexArr ={}
		curParsedArr = {}
		curCombineArr = {}
	end
	--[[解析文本]]
	function Parser:parse(str)
		if type(str)=="string" then
			self:clear()
			self:divide(str)
			self:parseSection()
			self:combine()
		end
		return curCombineArr
	end
	--[[重组数据]]
	function Parser:combine()
		local curRemain=curParsedArr[self["PARSE_REMAIN"]]
		curCombineArr=string.split(curRemain,"||")
		local j=1
		while (j-#curCombineArr<=0) do
			if curCombineArr[j]=="" then
				table.remove(curCombineArr,j)
				j=1
			else
				j=j+1
			end
		end
		local parsedCount = #curParsedArr
		if parsedCount>0 then
			table.sort(curParsedArr,function(v1,v2) return v1.index<v2.index end)
		end
		for k=1,#curCombineArr do
			if curCombineArr[k]~="MrC" then
				local normalFont = {}
				normalFont["type"]=self["PARSE_LABEL"].FONT
				normalFont["text"]=curCombineArr[k]
				curCombineArr[k] = normalFont
			else
				curCombineArr[k]=self:deepCopy(curParsedArr[1])
				table.remove(curParsedArr,1)
			end
		end
		return curCombineArr
	end
	--[[解析数据分组]]
	function Parser:parseSection()
		for label,section in pairs(curSectionsArr) do
			if label~=self["PARSE_REMAIN"] then
				if type(section)=="table" then
					local indexArr = curSectionsIndexArr[string.format(self["PARSE_INDEX"],label)]
					for i=1,#section do --有序的
						local font=section[i]
						local curFont={}
						local attrs=self:getAttr(font)
						local text=self:getInnerText(font)
						curFont["type"]=label
						if text~="" then
							curFont["text"]=text
						end
						if type(attrs)=="table" then
							for key,attr in pairs(attrs) do
								curFont[key]=attr                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
							end
						end
						curFont["index"] = indexArr[i]
						table.insert(curParsedArr,curFont)
					end
				end	
			else
				curParsedArr[label]=section
			end
		end
	end
	--[[解析颜色属性]]
	function Parser:parseAttrColor(colorStr)
		local c3bArr = {}
		if type(colorStr) == "string" then
			if #colorStr== 6 then
				c3bArr = self:hexToC3B(colorStr)
			elseif #colorStr == 7 and string.sub(colorStr,1,1)=="#" then
				c3bArr = self:hexToC3B(string.sub(colorStr,2))
			end
		end
		return c3bArr
	end
	--[[获取属性]]
	function Parser:getAttr(str)
		local curAttributeArr ={}
		if type(str)=="string" then
			for attr,value in string.gfind(str,self["FORMAT_ATTRIBUTE_PATTERN"]) do
				if attr == self["PARSE_ATTRIBUTE"].COLOR then
					curAttributeArr[self["PARSE_ATTRIBUTE"].COLOR]=self:parseAttrColor(value)
				elseif attr == self["PARSE_ATTRIBUTE"].SIZE then
					curAttributeArr[self["PARSE_ATTRIBUTE"].SIZE]=value
				elseif attr == self["PARSE_ATTRIBUTE"].SRC then
					curAttributeArr[self["PARSE_ATTRIBUTE"].SRC]=value
				elseif attr == self["PARSE_ATTRIBUTE"].IS_NEW_LINE then
					curAttributeArr[self["PARSE_ATTRIBUTE"].IS_NEW_LINE]=true
				elseif attr == self["PARSE_ATTRIBUTE"].IS_LINE then
					curAttributeArr[self["PARSE_ATTRIBUTE"].IS_LINE]=true
				elseif attr == self["PARSE_ATTRIBUTE"].IS_ALL_LINE then
					curAttributeArr[self["PARSE_ATTRIBUTE"].IS_ALL_LINE]=true
				elseif attr == self["PARSE_ATTRIBUTE"].ALIGN_H then
					curAttributeArr[self["PARSE_ATTRIBUTE"].ALIGN_H]=value
				elseif attr == self["PARSE_ATTRIBUTE"].ALIGN_V then
					curAttributeArr[self["PARSE_ATTRIBUTE"].ALIGN_V]=value
				elseif attr == self["PARSE_ATTRIBUTE"].FONT_NAME then
					curAttributeArr[self["PARSE_ATTRIBUTE"].FONT_NAME]=value
				end
			end
		end
		return curAttributeArr
	end
	--[[获取文本内容]]
	function Parser:getInnerText(str)
		local curInnerText=""
		if type(str)=="string" then
			curInnerText = string.match(str,self["FORMAT_INNER_TEXT_PATTERN"])
		end
		return curInnerText
	end
	--[[根据指定标签划分文本内容]]
	function Parser:divide(originStr)
		curSectionsArr["remain"]=""
		if type(originStr)=="string" then
			local curRemain=originStr
			for _,labelType in pairs(self["PARSE_LABEL"]) do
				local prefixLabel = self:getLabel(self["FORMAT_PREFIX_LABEL_PATTERN"],labelType)
				local suffixLabel = self:getLabel(self["FORMAT_SUFFIX_LABEL_PATTERN"],labelType)
				local pattern = string.format("%s[^%s]*%s",prefixLabel,string.sub(prefixLabel,1,1),suffixLabel)
				curSectionsArr[labelType]={}
				local labelTypeIndex = string.format(self["PARSE_INDEX"],labelType)
				curSectionsIndexArr[labelTypeIndex]={}
				for w in string.gfind(originStr,pattern) do
					table.insert(curSectionsArr[labelType],w)
					w=self:filter(w)
					local labelIndex = string.find(originStr,w)
					table.insert(curSectionsIndexArr[labelTypeIndex],labelIndex)
					curRemain = string.gsub(curRemain,w,self["PARSE_HOLDPLACE"])
				end
				curSectionsArr["remain"]=curRemain
			end
		end
	end
	--[[循环执行string.find(..)，得到对应索引数组]]
	function Parser:findingLoop(originStr,label)
		local curIndexArr ={}
		if type(originStr)=="string" and type(label)=="string" then
			local searchIndex=1
			local foundIndex = 1
			while(nil ~= foundIndex) do
				foundIndex = string.find(originStr,label,searchIndex)
				if nil ~= foundIndex and foundIndex-searchIndex>=0 then
					table.insert(curIndexArr,foundIndex)
					searchIndex = foundIndex+1
				end
			end
		end
		return curIndexArr
	end
	--[[获得标签]]
	function Parser:getLabel(format,labelType)
		local result=""
		if type(format)=="string" and type(labelType)=="string" then
			result = string.format(format,labelType)
		end
		return result
	end
	--[[十六进制颜色值转c3b]]
	function Parser:hexToC3B(hexColor)
		local decArr={}
		if type(hexColor)=="string" and #hexColor==6 then
			for i=1,#hexColor,2 do
				local hexValue=string.format("0x%s",string.sub(hexColor,i,i+1))
				local decValue=string.format("%d",hexValue)
				table.insert(decArr,decValue)
			end
		end
		return decArr
	end
	--[[对象深度复制]]
	function Parser:deepCopy(obj)
		local result={}
		if type(obj)~="table" then
			result=obj
		else
			for k,v in pairs(obj) do
				result[""..k]=self:deepCopy(v)
			end
		end
		return result
	end
	--[[模式匹配时内容转义]]
	function Parser:filter(desStr)
		local result=desStr
		if type(desStr)=="string" then
			result=string.gsub(result,"%[","%%%[")
			result=string.gsub(result,"%]","%%%]")
			result=string.gsub(result,"%+","%%%+")
			result=string.gsub(result,"%-","%%%-")
			result=string.gsub(result,"%*","%%%*")
			result=string.gsub(result,"%?","%%%?")
			result=string.gsub(result,"%(","%%%(")
			result=string.gsub(result,"%)","%%%)")
			--TODO
		end
		return result 
	end
-------------------------------------------

-- 返回该插件所扩展的基础类型。
function container.GetBaseType()
    return 'Text'
end

-- 返回这个包含 CreateCustomNode 和 GetBaseType 方法的表。
return container