-- 新建一个 table，包含 CreateCustomNode 和 GetBaseType 方法。
local container = {}

-- 新建根节点 Node，目前这个方法的名字为固定的，必须为 CreateCustomNode。
-- 方法的最后一句必须是一个 return 语句，把新建的结点返回。
function container.CreateCustomNode(tableType)
    local root = nil
    if not tableType then tableType=0 end
    if tableType == 0 then
        root = ccui.Layout:create()
    else
        root = ccui.PageView:create()
    end
    root._tableType = tableType

    root._requestDoLayout = root.requestDoLayout
    function root:requestDoLayout()
        root:_requestDoLayout()
        if root._tableType==0 then
            container.RefreshLayout(root)
        end
    end
    if __NotEditor then
        root._addChild = root.addChild
        function root:addChild(child, localZOrder, name)
            local n = #root._child
            root._child[n+1] = child
            if root._tableType==1 then
                container.RefreshPageLayout(root, n+1)
            else
                -- for 自适应布局 -----
                local __ui_layout = child:getComponent('__ui_layout')
                if not __ui_layout then
                    __ui_layout = ccui.LayoutComponent:bindLayoutComponent(child)
                end
                __ui_layout:setHorizontalEdge(1)
                __ui_layout:setVerticalEdge(2)
                child.__ui_layout = __ui_layout
                ------------------------
                root:_addChild(child)
                container.RefreshLayout(root)
            end
        end
        root._removeAllChildren = root.removeAllChildren
        function root:removeAllChildren()
            if root._tableType==1 then
                root._child = {}
                root:removeAllPages()
            else
                root._child = {}
                root:_removeAllChildren()              
            end
        end
        function root:pushBackCustomItem(child, localZOrder, name)
            root:addChild(child, localZOrder, name)
        end
    end
    root:setContentSize(200,200)

    container.init(root)
    root._child = {}
    return root
end

-- 返回该插件所扩展的基础类型。
function container.GetBaseType()
    return 'Panel'
end

function container.init(root)
    root._marginX = root._marginX or 50
    root._marginY = root._marginY or 50
    root._marginLeft = root._marginLeft or 0
    root._marginRight = root._marginRight or 0
    root._marginTop = root._marginTop or 0
    root._marginBottom = root._marginBottom or 0 
end

function container.RefreshLayout(root)
    container.init(root)
    local cd = root:getChildren()
    local size = root:getContentSize()
    if #cd<=0 then
        return
    end

    local itemsize = cd[1]:getContentSize()
    local marX = root._marginX + itemsize.width
    local marY = root._marginY + itemsize.height
    
    -- count num per row
    local numX = math.floor((size.width-itemsize.width-root._marginLeft-root._marginRight)/marX) + 1
    local lenX = (size.width-itemsize.width-root._marginLeft-root._marginRight) % marX
    numX = numX>0 and numX or 1
    -- count content height
    
    if root._tableType==0 then
        local ch = math.floor((#cd-1)/numX)*marY+itemsize.height+root._marginTop+root._marginBottom
        size.height = ch
        root:setContentSize(size)
    end

    local y0 = root._marginTop
    local x0 = root._marginLeft
    local curX = 0
    local cnt = #cd
    local dx = 0
    local dx2 = 0
    local modCnt = cnt % numX
    if root._horizontal then
        if root._horizontal==2 then
            dx = lenX/2
            dx2 = lenX/2 + (numX - (((cnt-1) % numX)+1))*marX/2
        elseif root._horizontal==1 then
            dx = lenX
            dx2 = lenX + (numX - (((cnt-1) % numX)+1))*marX    
        end       
    end
    for i=1,cnt do
        local ix = ((i-1) % numX)
        local iy = math.floor((i-1)/numX)
        local px = x0 + ix*marX
        local py = y0 + iy*marY

        if cnt-i+1<= modCnt then
            dx = dx2
        end
        cd[i].__ui_layout:setPosition(cc.p(px + dx , size.height - py - itemsize.height))
    end
    return true
end

function container.RefreshPageLayout(root, idx)
    local cd = root._child
    if #cd==0 then
        return
    end
    local size = root:getContentSize()
    local itemsize = root._child[1]:getContentSize()
    local marX = root._marginX + itemsize.width
    local marY = root._marginY + itemsize.height
    -- count num per page
    local numX = math.floor((size.width+1-itemsize.width-root._marginLeft-root._marginRight)/marX) + 1
    numX = numX>0 and numX or 1
    local numY = math.floor((size.height+1-itemsize.height-root._marginTop-root._marginBottom)/marY) + 1
    numY = numY>0 and numY or 1
    local numXY = numX*numY

    local curPage = math.floor((idx-1)/numXY)
    local layout = root:getPage(curPage)
    if layout==nil then
        layout = ccui.Layout:create()
        layout:setCascadeColorEnabled(true)
        layout:setCascadeOpacityEnabled(true)
        root:addPage(layout)
    end
    
    local idxInPage = ((idx-1)%numXY)
    local nx = idxInPage%numX
    local ny = math.floor(idxInPage/numX)

    local px = nx*marX + root._marginLeft
    local py = ny*marY + itemsize.height + root._marginTop
    layout:addChild(cd[idx])
    cd[idx]:setPosition(px, size.height - py)
       
    return true
end

function container.SetItemsMarginX(root, margin)
    root._marginX = margin
end
function container.SetItemsMarginY(root, margin)
    root._marginY = margin
    if not __NotEditor then
        container.RefreshLayout(root)
    end
end

function container.SetLeftMargin(root, margin)
    root._marginLeft = margin
end
function container.SetTopMargin(root, margin)
    root._marginTop = margin
    if not __NotEditor then
        container.RefreshLayout(root)
    end
end
function container.SetRightMargin(root, margin)
    root._marginRight = margin
end
function container.SetBottomMargin(root, margin)
    root._marginBottom = margin
    if not __NotEditor then
        container.RefreshLayout(root)
    end
end

function container.SetTableType(root, tableType)
    root._tableType = tableType
    if not __NotEditor then
        container.RefreshLayout(root)
    end
end

function container.SetHorizontal(root, horizontal)
    root._horizontal = horizontal
    if not __NotEditor then
        container.RefreshLayout(root)
    end
end

-- 返回这个包含 CreateCustomNode 和 GetBaseType 方法的表。
return container