local Line = class('Line', function() return gl.glNodeCreate() end )


function Line:ctor()
	self.vertexPositionBuffer = {}
    self.vertexTextureBuffer  = {}
    self.shader = cc.ShaderCache:getInstance():getGLProgram("ShaderPositionTextureColor")
    self.Point_Count = 0
    self.stroke = 10
end

function Line:setTexture(texname, color)
	self.texture1 = cc.Director:getInstance():getTextureCache():addImage(texname)
    self.color1 = color
    self.vertexColorBuffer1  = {}
end
function Line:setTexture2(texname, color)
	self.texture2 = cc.Director:getInstance():getTextureCache():addImage(texname)
    self.color2 = color
    self.vertexColorBuffer2  = {}    
end

function Line:setStroke(val)
    self.stroke = val
end

function Line:setRepeat()
    self.texture1:setTexParameters(gl.LINEAR, gl.LINEAR, gl.REPEAT, gl.REPEAT)
    if self.texture2 then
        self.texture2:setTexParameters(gl.LINEAR, gl.LINEAR, gl.REPEAT, gl.REPEAT)
    end
    self.bRepeat = true
end

local function convertVertexsToArray(vertices)
	local ret = {}
	for i=1, #vertices do
		ret[i*2-1] = vertices[i].x
		ret[i*2] = vertices[i].y
	end
	return ret
end

local function vertexLineToPolygon(points, stroke, offset, nuPoints)
	nuPoints = nuPoints+offset
	if nuPoints<=1 then return end

	stroke = stroke*0.5

	local vertices = {}
	local strokeT = 1

	local idx
	local nuPointsMinus = nuPoints-1

	for i=offset, nuPoints do
		idx = i*2-1
		local p1 = points[i]
		local perpVector

		if i==1 then
            perpVector = cc.pPerp(cc.pNormalize(cc.pSub(p1 , points[i+1])))
            strokeT = stroke
        elseif i == nuPoints then
        	perpVector = cc.pPerp(cc.pNormalize(cc.pSub(points[i-1] , p1)))
        	strokeT = stroke
        else
            local p2 = points[i+1]
            local p0 = points[i-1]

            local p2p1 = cc.pNormalize(cc.pSub(p2 , p1))
            local p0p1 = cc.pNormalize(cc.pSub(p0 , p1))

            -- Calculate angle between vectors
            local angle = math.acos( cc.pDot(p2p1, p0p1))
            if angle < math.rad(10) then
                perpVector = cc.pPerp(cc.pNormalize(cc.pMidpoint(p2p1, p0p1)))
            elseif angle < math.rad(170) then
                perpVector = cc.pNormalize(cc.pMidpoint(p2p1, p0p1))
            else
                perpVector = cc.pPerp(cc.pNormalize(cc.pSub(p2, p0)))
            end
            local p = cc.pDot(p0p1, perpVector)
        	strokeT = stroke/math.sqrt(1-p*p)
        end
       
        perpVector = cc.pMul(perpVector , strokeT)

        vertices[idx] = cc.pAdd(p1,  perpVector)
        vertices[idx + 1] = cc.pSub(p1, perpVector)
	end
	-- Validate vertexes
	--offset = (offset==0) ? 0 : offset-1;
	offset = (offset<=1) and 1 or offset-1

    for i = offset, nuPointsMinus do
        local idx = i*2-1
        local idx1 = idx+2;

        local p1 = vertices[idx]
        local p2 = vertices[idx+1]
        local p3 = vertices[idx1]
        local p4 = vertices[idx1+1]

        local s, t
        local fixVertex, s, t = cc.pIsLineIntersect(p1, p4, p2, p3, s, t)
        if not fixVertex then
            if s<0 or s>1 then
                fixVertex = true
            end
        else
        	if s>0 and s<1 then
        		fixVertex = false
        	end
        end
        if fixVertex then
            vertices[idx1] = p4
            vertices[idx1+1] = p3
        end
    end
    return vertices
end
function Line:init(points)
	self.points = points
    self.vertexPositionBuffer.buffer_id = gl.createBuffer()
    gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexPositionBuffer.buffer_id)
    
    self.Point_Count = #(self.points)
    local vertices = vertexLineToPolygon(self.points, self.stroke, 1, self.Point_Count-1)
    vertices = convertVertexsToArray(vertices)

    gl.bufferData(gl.ARRAY_BUFFER,table.getn(vertices),vertices,gl.STATIC_DRAW)

    self.vertexTextureBuffer.buffer_id = gl.createBuffer()
    gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexTextureBuffer.buffer_id)
    local texcoords = {}
    local cnt = self.Point_Count-1
    local sc = self.bRepeat and cc.pGetDistance(points[1], points[#points])/128 or 1
    for i=0, cnt do
    	local tx = i/cnt
    	texcoords[i*4+1] = tx*sc
    	texcoords[i*4+2] = 0
    	texcoords[i*4+3] = tx*sc
    	texcoords[i*4+4] = 1
    end
    gl.bufferData(gl.ARRAY_BUFFER,table.getn(texcoords),texcoords,gl.STATIC_DRAW)
    gl.bindBuffer(gl.ARRAY_BUFFER,0)

    -- color1
    self.vertexColorBuffer1.buffer_id = gl.createBuffer()
    gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexColorBuffer1.buffer_id)
    local colors = {}
    local c = self.color1
    for i=0, cnt*2+1 do
    	colors[i*4+1] = c.r
    	colors[i*4+2] = c.g
    	colors[i*4+3] = c.b
    	colors[i*4+4] = c.a
    end

    gl.bufferData(gl.ARRAY_BUFFER, table.getn(colors), colors, gl.STATIC_DRAW)
    gl.bindBuffer(gl.ARRAY_BUFFER, 0)

    if self.texture2 then
        -- color2
        self.vertexColorBuffer2.buffer_id = gl.createBuffer()
        gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexColorBuffer2.buffer_id)
        local colors2 = {}
        local c = self.color2
        for i=0, cnt*2+1 do
    	    colors2[i*4+1] = c.r
    	    colors2[i*4+2] = c.g
    	    colors2[i*4+3] = c.b
    	    colors2[i*4+4] = c.a
        end

        gl.bufferData(gl.ARRAY_BUFFER, table.getn(colors2), colors2, gl.STATIC_DRAW)
        gl.bindBuffer(gl.ARRAY_BUFFER, 0)    
    end

    self:registerScriptDrawHandler(self:draw())
end

function Line:setPercent(percent)
    local total = 0
    local list = { 0 }
    for i=1, #self.points-1 do
        local v1 = self.points[i]
        local v2 = self.points[i+1]
        local l = cc.pGetDistance(v1,v2)
        total = total + l
        list[#list+1] = l
    end
    
    percent = percent<=1 and percent or 0.99999
    local totalPercent = total*percent

    local pos = #list
    for i=1, #list do
        if totalPercent<list[i] then
            pos = i
            break
        end
        totalPercent = totalPercent - list[i]
    end

    local points = {}
    for i=1, pos do
        points[i] = util.tableCopy(self.points[i])
    end

    points[pos] = cc.pLerp(points[pos-1], points[pos], totalPercent/list[pos])

    gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexPositionBuffer.buffer_id)
    
    self.Point_Count = #(points)
    local vertices = vertexLineToPolygon(points, self.stroke, 1, self.Point_Count-1)
    vertices = convertVertexsToArray(vertices)

    gl.bufferData(gl.ARRAY_BUFFER,table.getn(vertices),vertices,gl.STATIC_DRAW)
end

function Line:Clear()
	self:unregisterScriptDrawHandler(self:draw())
end

function Line:draw()
	return function(transform, transformUpdated)
		if nil ~= self.shader then
	        self.shader:use()
	        self.shader:setUniformsForBuiltins(transform)

            gl.blendFunc(1, 771)

	        gl.bindTexture(gl.TEXTURE_2D, self.texture1:getName())
	        gl.glEnableVertexAttribs(bit._or(cc.VERTEX_ATTRIB_FLAG_TEX_COORDS, bit._or(cc.VERTEX_ATTRIB_FLAG_POSITION, cc.VERTEX_ATTRIB_FLAG_COLOR) ))
			--gl.glEnableVertexAttribs(cc.VERTEX_ATTRIB_FLAG_COLOR or cc.VERTEX_ATTRIB_FLAG_POSITION or cc.VERTEX_ATTRIB_FLAG_TEX_COORDS)

	        gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexPositionBuffer.buffer_id)
	        gl.vertexAttribPointer(cc.VERTEX_ATTRIB_POSITION,2,gl.FLOAT,false,0,0)


	        gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexTextureBuffer.buffer_id)
	        gl.vertexAttribPointer(cc.VERTEX_ATTRIB_TEX_COORD,2,gl.FLOAT,false,0,0)

			gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexColorBuffer1.buffer_id)
            gl.vertexAttribPointer(cc.VERTEX_ATTRIB_COLOR, 4, gl.FLOAT, false, 0, 0)

	        gl.drawArrays(gl.TRIANGLE_STRIP,0,self.Point_Count*2)


            if self.texture2 then
 			    gl.bindTexture(gl.TEXTURE_2D, self.texture2:getName())
	            gl.glEnableVertexAttribs(bit._or(cc.VERTEX_ATTRIB_FLAG_TEX_COORDS, bit._or(cc.VERTEX_ATTRIB_FLAG_POSITION, cc.VERTEX_ATTRIB_FLAG_COLOR) ))

	            gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexPositionBuffer.buffer_id)
	            gl.vertexAttribPointer(cc.VERTEX_ATTRIB_POSITION,2,gl.FLOAT,false,0,0)

	            gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexTextureBuffer.buffer_id)
	            gl.vertexAttribPointer(cc.VERTEX_ATTRIB_TEX_COORD,2,gl.FLOAT,false,0,0)

			    gl.bindBuffer(gl.ARRAY_BUFFER, self.vertexColorBuffer2.buffer_id)
                gl.vertexAttribPointer(cc.VERTEX_ATTRIB_COLOR, 4, gl.FLOAT, false, 0, 0)

	            gl.drawArrays(gl.TRIANGLE_STRIP,0,self.Point_Count*2)

            end

	        gl.bindTexture(gl.TEXTURE_2D,0)
	        gl.bindBuffer(gl.ARRAY_BUFFER,0)
	    end
	end
end

return Line