cc.exports.ctr = ctr or {}
local updator = {}
--if device.platform == "windows" then
--    if cc.FileUtils:getInstance():isFileExist(".project") then
--        updator.isTest = true
--    end
--end
updator.isTest = CC_NO_UPDATOR
local mainManifest = "res/Manifests/project.manifest"
local storagePath = CC_DOWNLOAD_PATH
local am
local path = cc.FileUtils:getInstance():getWritablePath()..storagePath
local lockFile = string.format("%s/lock",path)

local isLoaded = false
local onUpdateEvent = nil

function updator.init()
    updator.checkUpdating()
    am = cc.AssetsManagerEx:create(mainManifest, path)
    am:retain()
    isLoaded = am:getLocalManifest():isLoaded()
    local listener = cc.EventListenerAssetsManagerEx:create(am,onUpdateEvent)
    cc.Director:getInstance():getEventDispatcher():addEventListenerWithFixedPriority(listener, 1)
end

function updator.checkUpdating()
    print("【开启了热更新……】")
    if io.exists(lockFile) then
        -- print("-------------------->>>>>removeDirectory:",cc.FileUtils:getInstance():removeDirectory(ss))
        if cc.FileUtils:getInstance():removeFile(string.format("%s/version.manifest",path)) and cc.FileUtils:getInstance():removeFile(string.format("%s/project.manifest",path)) then
            cc.FileUtils:getInstance():removeFile(lockFile)
            print("【检测到上次尚未更新完全，重启更新操作……】")
        else
            if io.exists(lockFile) then
                cc.FileUtils:getInstance():removeFile(lockFile)
            end
            print("【检测到上次尚未更新完全，重启更新发生异常！】")
        end
    else
        io.writefile(lockFile, tostring(os.time()))
    end
end

function updator.reload()
    for k,v in pairs(package.loaded) do
        if k~='json' then
            package.loaded[k] = nil
        end
    end
    require('app.initcustom').reload()
end

function updator.needUpdate()
    return ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.NEW_VERSION_FOUND
end

function updator.finished()
    return ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.UPDATE_FINISHED or 
    ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.ALREADY_UP_TO_DATE
end

function updator.isError()
    return ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_NO_LOCAL_MANIFEST or
        ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_DOWNLOAD_MANIFEST or 
        ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_PARSE_MANIFEST or
        ctr.updator.eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_UPDATING
end

function onUpdateEvent(ev)
    local eventCode = ev:getEventCode()
    if eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_NO_LOCAL_MANIFEST then
        print("No local manifest file found, skip assets update.")
        ctr.updator.loading = false                
    elseif  eventCode == cc.EventAssetsManagerEx.EventCode.UPDATE_PROGRESSION then
        local assetId = ev:getAssetId()
        local percent = ev:getPercent()
        local strInfo = ""

        if assetId == cc.AssetsManagerExStatic.VERSION_ID then
            strInfo = string.format("Version file: %d%%", percent)
        elseif assetId == cc.AssetsManagerExStatic.MANIFEST_ID then
            strInfo = string.format("Manifest file: %d%%", percent)
        else
            strInfo = string.format("%d%%", percent)
        end
        ctr.updator.percent = percent
        print(strInfo)
    elseif eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_DOWNLOAD_MANIFEST or 
        eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_PARSE_MANIFEST then
        print("Fail to download manifest file, update skipped. "..eventCode)
        --ctr.updator.loading = false
    elseif eventCode == cc.EventAssetsManagerEx.EventCode.ALREADY_UP_TO_DATE or 
        eventCode == cc.EventAssetsManagerEx.EventCode.UPDATE_FINISHED then
        print("Update finished.")
        ctr.updator.loading = false
        if io.exists(lockFile) then
            cc.FileUtils:getInstance():removeFile(lockFile)
        end
    elseif eventCode == cc.EventAssetsManagerEx.EventCode.ERROR_UPDATING then
        print("Asset ", ev:getAssetId(), ", ", ev:getMessage())
        ctr.updator.loading = false
    elseif eventCode == cc.EventAssetsManagerEx.EventCode.NEW_VERSION_FOUND then
        if ctr.updator.needMajorUpdate() then
            if util.isIOS() then
                util.showDialog(nil, 'Common.NeedUpdateDialog')     --跳appstore
                am:release()                                        --删除am 停掉下载操作
                return
            elseif util.isAndroid() then
                --todo 安卓更新
            else
                --默认不做更新操作
            end
        end
        print('Found new version, ready to update!')
    end
           
    ctr.updator.eventCode = eventCode
    event.dispatch('onUpdatorEvent' )
end

function updator.check()
    ctr.updator.eventCode = nil
    ctr.updator.isComplete = false
    if not isLoaded then
    	print('Fail to update assets, step skipped.')
    else
        am:checkUpdate()
    end
end

function updator.doload()
    

	-- TODO 
	ctr.updator.loading = true
	local function info(msg)
		event.dispatch("onUpdatorInfo", msg)
		--print(msg)
	end

    -- ¿ª·¢Ä£Ê½²»ÏÂÔØ×ÊÔ´
    if updator.isTest then
        ctr.updator.isComplete = true
        ctr.updator.eventCode = cc.EventAssetsManagerEx.EventCode.UPDATE_FINISHED
        event.dispatch('onUpdatorEvent' )
        return
    end

	if not isLoaded then
    	info('Fail to update assets, step skipped.')
    else
        util.delay(0.1, function()
        am:update()
        end)
    end
end

function updator.isLoading()
    return ctr.updator.loading
end

--获取本地版本号
function updator.getLocalVersion()
    return am:getLocalManifest():getVersion()
end

--获取远程版本号
function updator.getRemoteVersion()
    return am:getRemoteManifest():getVersion()
end

--判断是否需要进行大版本更新（appstore更新）
function updator.needMajorUpdate()
    local localVersion = updator.getLocalVersion()
    local remoteVersion = updator.getRemoteVersion()
    local localMajorVersion = string.split(localVersion, '.')
    local remoteMajorVersion = string.split(remoteVersion, '.')

    --判断前面两位是否相同
    if localMajorVersion[0] ~= remoteMajorVersion[0] then
        return true
    end

    if localMajorVersion[1] ~= remoteMajorVersion[1] then
        return true
    end

    return false
end

ctr.updator = updator
return ctr.updator