local SDKApplePay=class("SDKApplePay",sdkManager:getSDK("sdkBase"))
	
	SDKApplePay["id"]="applepay"
	SDKApplePay["isRequested"]=false
	SDKApplePay["requestedProducts"]={}
	SDKApplePay["cmd"]={
		PAY=1001, --发起商品购买请求
		REQUEST_PRODUCTS=1002, --请求可购买商品列表
		RESTORE_PURCHASES=1003,
		PAY_REBORN=1004,--特殊处理复活购买(没有进行物品列表)

		CB_REQUEST_PRODUCTS=2020, --返回有效商品列表
		
		CB_PAY_SUCCESS=2010, --支付成功
		CB_PAY_FAILED=2011, --支付失败
		CB_RESTORE_PURCHASES_SUCCESS=2004; --恢复购买成功
		CB_RESTORE_PURCHASES_FAILED=2005; --恢复购买失败
	}

	--[[获取当前商品验证状态]]
	function SDKApplePay:isRequestedProducts()
		return self.isRequested
	end
	--[[请求有效的商品列表]]
	function SDKApplePay:requestProducts(productsList,callback)
		if self:isIOS() then
			if type(productsList) == "table" then
				if false == self:isRequestedProducts() then
					if type(callback) == "function" then
						self:setCallback(self.cmd.REQUEST_PRODUCTS,callback)
					end
					sdkManager:luaBridge(self:createParameter(self.id,self.cmd.REQUEST_PRODUCTS,productsList))
				end
			end
		end
	end
	--[[刷新商城物品列表]]
	function SDKApplePay:processRequestProducts(products)
		if self:isIOS() then
			if type(products) == "table" and table.nums(products)>0 then
				self.isRequested = true
				self.requestedProducts = products
				local cbFunc = self:getCallback(self.cmd.REQUEST_PRODUCTS)
				if type(cbFunc) == "function" then
					cbFunc(self.isRequested)
				end
			end
		end
	end
	--[[购买内购商品]]
	function SDKApplePay:pay(productKey,callback,isLocalValidate)
		if self:isIOS() then
		    if type(productKey) == "string" then
		    	if type(callback) == "function" then
		    		self:setCallback(self.cmd.PAY,callback)
		    	end
		        local product = {}
		        product["productKey"]=productKey
		        product["isLocalValidate"]=(type(isLocalValidate)=="boolean" and isLocalValidate or false)
		        sdkManager:luaBridge(self:createParameter(self.id,self.cmd.PAY,product))
	    	end
		end
	end
	--[[用户购买成功处理]]
	function SDKApplePay:processPay(isSuccess,info)
		if self:isIOS() then
			local cbFunc = self:getCallback(self.cmd.PAY)
			if type(cbFunc) == "function" and type(isSuccess) == "boolean" then
				cbFunc(isSuccess,info)
			end
		end
	end
	--[[获取验证后的商品信息]]
	function SDKApplePay:getValidatedProduct(productKey)
		local productInfo = {}
		if self:isIOS() then
			if type(productKey) == "string" then
				productInfo = self.requestedProducts[productKey]
				if productInfo then
					productInfo["key"] = productKey
				end
			end
		end
		return productInfo
	end
	--[[请求恢复购买]]
	function SDKApplePay:requestRestorePurchases()
		if self:isIOS() then
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.RESTORE_PURCHASES))
		end
	end
	--[[用户恢复购买成功处理]]
	function SDKApplePay:processRestorePurchasesSuccess(productKey)
		if self:isIOS() then
			if type(productKey)=="string" then
				ctr.player.shopRestorePurchasesSuccess(productKey)
			end
		end
	end
	--[[购买回调处理]]
	function SDKApplePay:handler(response)
		if self:isIOS() then
			if type(response)=="table" then
				local cmd=response["cmd"]
				if type(cmd)=="number" then
					if cmd==self.cmd.CB_REQUEST_PRODUCTS then
						printInfo("【得到有效购买列表！】")
						self:processRequestProducts(response.param)
					elseif cmd==self.cmd.CB_PAY_SUCCESS then
						printInfo("【用户支付完成，购买成功！】")
						--param=="物品identifier"
						self:processPay(true,response.param)
					elseif cmd==self.cmd.CB_PAY_FAILED or cmd==self.cmd.CB_RESTORE_PURCHASES_FAILED then
						printInfo("【用户支付中断，购买/恢复购买失败！】")
						--param=="yes" 用户取消
						self:processPay(false,response.param)
					elseif cmd==self.cmd.CB_RESTORE_PURCHASES_SUCCESS then
						printInfo("【用户恢复购买成功！】")
						self:processRestorePurchasesSuccess(response.param)
					end
				end
			end
		end
	end

return SDKApplePay