local SDKMrC = class("SDKMrC",sdkManager:getSDK("sdkBase"))

	SDKMrC["id"] = "mrc"
	SDKMrC["cmd"] = {
		GET_VERSION = 1001,--获取当前版本字符串
		GET_LANGUAGE = 1002,--获取当前语言
		SHOW_DIALOG_ALERT = 1003,--显示系统提示框
		SHOW_DETAIL_IN_STORE = 1004,--应用内跳转appStore
		SHOW_SYSTEM_SHARE = 1005,--打开系统分享
		SHOW_SYSTEM_ALBUM = 1006,--打开系统相册
		PROCESS_IMAGE = 1007,--处理本地图片(缩放\圆角处理等)

	    CB_SHOW_DIALOG_ALERT = 2030, --[回调]显示系统提示框
	    CB_SHOW_SYSTEM_SHARE = 2050, --[回调]打开系统分享
	    CB_SHOW_SYSTEM_ALBUM = 2060, --[回调]打开系统相册
	   	CB_PROCESS_IMAGE = 2070, --[回调]处理本地图片(缩放\圆角处理等)
	}

	--[[获取当前语言版本]]
	function SDKMrC:getLanguage()
		local result = "en"
		if self:isIOS() then
			result = sdkManager:luaBridge(self:createParameter(self.id,self.cmd.GET_LANGUAGE)).language
		elseif self:isAndroid() then
			result = sdkManager:luaBridge(self:createParameter(self.id,self.cmd.GET_LANGUAGE))
			print("language = " .. result)
		end
		printInfo("【%s】当前语言:%s",self.id,result)
		return result
	end
	--[[获取当前Version版本]]
	function SDKMrC:getVersion()
		local result = ""
		if self:isIOS() then
			local versionInfo = sdkManager:luaBridge(self:createParameter(self.id,self.cmd.GET_VERSION))
			if type(versionInfo) == "table" and versionInfo.version then
				result = versionInfo.version
			end
		elseif self:isAndroid() then
			result = sdkManager:luaBridge(self:createParameter(self.id,self.cmd.GET_VERSION))
		end
		printInfo("【%s】当前版本:%s",self.id,result)
		return result
	end
	--[[打开Android系统确认框]]
	--[[param类型为table，其有效字段:
				title 退出提示框标题
			  message 退出提示框内容
	   positiveButton 确认按钮文字
	   nagetiveButton 取消按钮文字
	]]
	function SDKMrC:showAlertDialog(callback,param)
		if self:isAndroid() then
			if type(callback) == "function" then
				self:setCallback(self.cmd.SHOW_DIALOG_ALERT,callback)
			end
			if type(param) ~= "table" then
				param = {["title"]="Warning",["message"]="Are you sure to quit the game?",["positiveButton"]="YES",["negativeButton"]="NO"}
			else
				param["title"] = self:checkString(param.title,"Warning")
				param["message"] = self:checkString(param.message,"Are you sure to quit the game?")
				param["positiveButton"] = self:checkString(param.positiveButton,"YES")
				param["negativeButton"] = self:checkString(param.negativeButton,"NO")
			end
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.SHOW_DIALOG_ALERT,param))
		end
	end
	--[[回调==打开Android系统确认框]]
	function SDKMrC:processShowAlertDialog(param)
		local cbFunc = self:getCallback(self.cmd.SHOW_DIALOG_ALERT)
		if type(cbFunc) == "function" then
			cbFunc(param)
		end
	end
	--[[从商店里打开App详细页]]
	--[[storePackage类型为string，其在iOS下此参数无效，在Android下，有效值为:
		com.android.vending 	Google Play
		com.tencent.android.qqdownloader 	应用宝
		com.qihoo.appstore 	360手机助手
		com.baidu.appsearch 	百度手机助
		com.xiaomi.market 	小米应用商店
		com.wandoujia.phoenix2 	豌豆荚
		com.huawei.appmarket 	华为应用市场
		com.taobao.appcenter 	淘宝手机助手
		com.hiapk.marketpho 	安卓市场
		cn.goapk.market 	安智市场
	]]
	function SDKMrC:showDetailInStore(storePackage)
		if self:isAndroid() then
			local param = {}
			param["storePackageName"] = self:checkString(storePackage,"com.android.vending")
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.SHOW_DETAIL_IN_STORE,param))
		elseif self:isIOS() then
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.SHOW_DETAIL_IN_STORE))
		end
	end
	--[[打开系统分享]]
	function SDKMrC:showSystemShare(tips,url,imagePath,callback)
		local param = {}
		param["tips"] = self:checkString(tips,"share tips")
		param["url"] = url--TODO 校验url格式
		param["imagePath"] = self:checkString(imagePath,nil)
		if type(callback) == "function" then
			self:setCallback(self.cmd.SHOW_SYSTEM_SHARE,callback)
		end
		if self:isIOS() then
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.SHOW_SYSTEM_SHARE,param))
		end
	end
	--[[回调==打开系统分享]]
	function SDKMrC:processShowSystemShare(isCompleted)
		local cbFunc = self:getCallback(self.cmd.SHOW_SYSTEM_SHARE)
		if type(cbFunc) == "function" then
			cbFunc(isCompleted=="true")
		end
	end
	--[[打开系统相册，可附带对选择的图片进行处理]]
	--[[可对图片进行的操作说明:
		process["round"] = true --对图片进行圆形裁切
		process["size"] = {width=200,height=400} --对图片进行尺寸调整
	]]
	function SDKMrC:showSystemAlbum(callback,process)
		if type(callback) == "function" then
			self:setCallback(self.cmd.SHOW_SYSTEM_ALBUM,callback)
		end
		local param = {}
		param["imagePath"] = device.writablePath
		param["imageName"] = string.format("local_%s.png",os.time())
		param["process"] = process
		if self:isIOS() then
			sdkManager:luaBridge(self:createParameter(self.id,self.cmd.SHOW_SYSTEM_ALBUM,param))
		end
	end
	--[[回调==打开系统相册]]
	function SDKMrC:processShowSystemAlbum(curPickedImageFullPath)
		local cbFunc = self:getCallback(self.cmd.SHOW_SYSTEM_ALBUM)
		if type(cbFunc) == "function" then
			cbFunc(curPickedImageFullPath)
		end
	end
	--[[对本地图片进行处理]]
	--[[可对图片进行的操作说明:
		param["round"] = true --对图片进行圆形裁切
		param["size"] = {width=200,height=400} --对图片进行尺寸调整
	]]
	function SDKMrC:doSomething2Image(imageFullPath,param,callback)
		if type(imageFullPath) == "string" then
			if type(callback) == "function" then
				self:setCallback(self.cmd.PROCESS_IMAGE,callback)
			end
			param = self:checkTable(param,nil)
			if nil ~= param then
				param["imageFullPath"] = imageFullPath
				sdkManager:luaBridge(self:createParameter(self.id,self.cmd.PROCESS_IMAGE,param))
			elseif type(callback) == "function" then
				callback(false)
			end
		end
	end
	--[[回调==对本地图片进行处理]]
	function SDKMrC:processDoSomething2Image(imageFullPath)
		local cbFunc = self:getCallback(self.cmd.PROCESS_IMAGE)
		if type(cbFunc) == "function" then
			cbFunc(imageFullPath)
		end
	end
	--[[回调处理]]
	function SDKMrC:handler(response)
		if sdkManager:isAndroid() or sdkManager:isIOS() then
			if type(response)=="table" then
				local cmd = response["cmd"]
				if type(cmd) == "number" then
					if cmd == self.cmd.CB_SHOW_DIALOG_ALERT then
						self:processShowAlertDialog(response.param)
					elseif cmd == self.cmd.CB_SHOW_SYSTEM_SHARE then
						self:processShowSystemShare(response.param)
					elseif cmd == self.cmd.CB_SHOW_SYSTEM_ALBUM then
						self:processShowSystemAlbum(response.param)
					elseif cmd == self.cmd.CB_PROCESS_IMAGE then
						self:processDoSomething2Image(response.param)
					end
				end
			end
		end
	end

return SDKMrC