local sdkManager = class("sdkManager")
	sdkManager["isInited"] = false

	function sdkManager:init(callback)
		if not self.isInited then
	        self:registerBridgeCallBack(handler(self,self.callBack))
	        if type(callback)=="function" then
	        	callback()
	    	end
	        self.isInited = true
	        printInfo("【SDK初始化完成】")
	    end
	end
	--[[SDK统一回调入口]]
	function sdkManager:callBack(param)
		print("-----------------------------------------------")
		print("-----------------------------------------------")
		print("-----------------------------------------------")

		if type(param) == "string" then
			param = json.decode(param)
		end
		dump(param)
		if type(param) == "table" then
			local id = param["id"]
			if type(id) == "string" then
				print("-------------------")
				self:handler(id, param)
			end
		end
	end
	--[[SDK的全局处理]]
	function sdkManager:handler(sdkName,param)
		if type(sdkName) == "string" and type(param) == "table" then
			for k,v in pairs(self) do
				if type(v) == "table" then
					if v.id == sdkName then
						print("lua sdk file found")
						print(v.id)
						dump(param)
						v:handler(param)
						break
					end
				end
			end
		end
	end
	--[[将需要使用的SDK进行注册]]
	function sdkManager:registerSDK(sdkName)
		if type(sdkName) == "string" then
			local curSDK = self:getSDK(sdkName,true)
			if curSDK then
				self[curSDK.id] = curSDK
				printInfo("【注册SDK成功】:%s",curSDK.id)
			end
		end
	end
	--[[获取指定SDK的引用]]
	function sdkManager:getSDK(sdkName,isInstance)
		local result = nil
		if type(sdkName) == "string" then
			result = require(string.format("app.extensions.sdk.entity.%s",sdkName))
		end
		if isInstance then
			return result.new()
		else
			return result
		end
	end
	--注册外部回调方法(默认为SDK的回调)
	function sdkManager:registerBridgeCallBack(callback)
	    if self:isIOS() and type(callback) == "function" then
	        local methodName = "registerScriptHandler"

            local luaoc = require "cocos.cocos2d.luaoc"
            local className = "LuaObjectCBridge"

	        local ok = luaoc.callStaticMethod(className,methodName,{["scriptHandler"] = callback })
	        if ok then
	            printInfo("【注册外部回调方法成功】")
	        else
	            printInfo("【注册外部回调方法失败】")
	        end
	    else
	        printInfo("【注册外部调用回调失败】")
	    end
	end
	--lua调用外部方法
	function sdkManager:luaBridge(args)
	    if self:isAndroid() then
	        local luaj = require "cocos.cocos2d.luaj"
	        local className = "org/cocos2dx/lua/AppActivity"
	        local sigs = "(Ljava/lang/String;)Ljava/lang/String;"
	        if type(args)=="table" then
	            args={json.encode(args)}
	        end
	        local ok,ret = luaj.callStaticMethod(className,"callJavaCommand",args,sigs)
	        if ok then
	            return ret
	        end
	    elseif self:isIOS() then
	        local luaoc = require "cocos.cocos2d.luaoc"
	        local className = "LuaObjectCBridge"
	        local ok,ret  = luaoc.callStaticMethod(className,"callObjCCommand",args)
	        if ok then
	            printInfo("调用callObjCCommand方法成功！")
	            return ret
	        else
	            printInfo("调用callObjCCommand方法失败！！！")
	        end
	    end
	end
	--[是否Android系统]
	function sdkManager:isAndroid()
		return device.platform=="android"
	end
	--[[是否iOS系统]]
	function sdkManager:isIOS()
		return (device.platform=="ios" or device.platform=="ipad")
	end

    function sdkManager:checkAdAvalible(adsInfo)
        if static.isInnerTest then
            return false
        end

        dump(adsInfo)

        local result = false
	    if type(adsInfo) == "table" and adsInfo["isAdsReady"] then
	    	result = (true == adsInfo["isAdsReady"] or "true" == adsInfo["isAdsReady"] )
        elseif type(json.decode(adsInfo, 1)) == "table" then
             local t = json.decode(adsInfo, 1)
             if t["isAdsReady"] then
                result = (t["isAdsReady"] == "true" or true == t["isAdsReady"])
             end
	    end
        return result
    end

cc.exports.sdkManager = sdkManager
return sdkManager