cc.exports.async = {}

---------------- AsyncFuncClass  -----
local AsyncFunc = {}
function AsyncFunc:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function AsyncFunc:call()
    return async.call(self)
end

--function AsyncFunc:_call(cb)
--end

function AsyncFunc:isOver()
	return self._val~=nil
end
function AsyncFunc:get()
	return unpack(self._val)
end
function AsyncFunc:set(...)
	self._val = {...}
end

function AsyncFunc:resume(...)
    if self.cb then
        local cb = self.cb
        self.cb = nil
        cb(...)
    end
end
---------------- AsyncFuncClass  -----

function async.create(func)
	local co = coroutine.create(func)
	local status, info = coroutine.resume(co, co)
    if not status then
        print(info)
    end
end
function async.call(func)
	local co = coroutine.running()

	func:_call( function(...) 
		func:set(...)
		local status, info = coroutine.resume(co, ...)
        if not status then
            print(info)
        end
	end)
	return coroutine.yield()
end
function async.callNoWait(func)
	--local co = coroutine.running()
	func:_call( function(...) 
		func:set(...)
--		local status, info = coroutine.resume(co, ...)
--        if not status then
--            print(info)
--        end
	end)
	--return coroutine.yield()
end
function async.delay(sec)
	local co = coroutine.running()
	local function delaycb()
		local status, info = coroutine.resume(co)
        if not status then
            print(info)
        end
	end
	util.delay(sec, delaycb)
	return coroutine.yield()
end

function async.waitForEvent(eventname)
	local co = coroutine.running()
	local listener = nil
	local function listenercb(ev)
		event.remove(listener)
		local status, info = coroutine.resume(co, ev._usedata)
        if not status then
            print(info)
        end
	end	
	listener = event.addEvent(eventname, listenercb, 1)
	return coroutine.yield()
end

function async.funcAnd(f1, f2)
	local af = AsyncFunc:new()
	function af:_call(cb)
		f1:_call(function(...)
			f1:set(...)
			if f2:isOver() then
				cb(...)
			end
		end)
		f2:_call(function(...)
			f2:set(...)
			if f1:isOver() then
				cb(...)
			end
		end)
	end
	return af
end

function async.funcOr(f1, f2)
	local af = AsyncFunc:new()
	function af:_call(cb)
		f1:_call(function(...)
			f1:set(...)
			if not f2:isOver() then
				cb(...)
			end
		end)
		f2:_call(function(...)
			f2:set(...)
			if not f1:isOver() then
				cb(...)
			end
		end)
	end
	return af
end

function async.bindEvent(eventname)
	local af = AsyncFunc:new()
	function af:_call(cb)
		local listener = nil
		local function listenercb(ev)
			event.remove(listener)
			af:set(ev._usedata)
			cb(ev._usedata)
		end	
		listener = event.addEvent(eventname, listenercb, 1)
	end
	return af	
end

function async.bindDelay(sec)
	return async.bind(util.delay, sec, async.callback)
end

function async.bindCall(f, ...)
    return async.call(async.bind(f, ...))
end

function async.bind(f, ...)
	local args = {...}
	local idx = -1
	for i=1,#args do
		if args[i]==async.callback then
			idx = i
		end
	end
	local af = AsyncFunc:new()
	function af:_call(cb)
        if f then
		    args[idx] = function(...)
			    af:set(...)
			    cb(...) 
		    end
		    f(unpack(args))
        else
            af.cb = cb
        end
	end
	return af
end

function async.bindAction(node, act)
	local af = AsyncFunc:new()
	function af:_call(cb)
		local function actioncb(...)
			af:set(...)
			cb(...)
		end
		local seq = cc.Sequence:create(act, cc.CallFunc:create(actioncb))
		node:runAction(seq)
		end
	return af	
end

function async.callback()
end



return async