cc.exports.audio = {}
local soundInfo = require('res/Sound/SoundList')
local playList = {}
audio.bMusicEnable = true
audio.bEffectEnable = true
audio.curMusic = nil
event.addEvent('applicationDidEnterBackground', function()
    audio.pauseMusic()
end, 1) 
event.addEvent('applicationWillEnterForeground', function()
    audio.resumeMusic()
end, 1) 

-- state 0-ready 1-play  2-pause 3-stop
local function getSoundInfo(key)
    if key==nil then return nil end

    local info0 = playList[key]
    if info0 then
        return info0
    end

    local info = {}
    local s = soundInfo.SoundList[key]
    if s then
        info.file = s
    else
        info.file = key
    end
    info.id = nil
    return info
end

function audio.preloadSound()
    for k, v in pairs(soundInfo.SoundList) do
        print("SOUND =" .. v)
        ccexp.AudioEngine:preload("Sound/"..v)
    end

end

local function playInfo(info)
    

    info.id = ccexp.AudioEngine:play2d('Sound/'..info.file)
    info.state = 1
    if info.callback then
        local callback = info.callback
        ccexp.AudioEngine:setFinishCallback(info.id, function()
            info.id = nil
            info.state = 3
            callback()
        end)
    end
    playList[info.file] = info
    return info.id
end

local function stopInfo(info)
    if info.id then
        ccexp.AudioEngine:stop(info.id)
        info.id = nil
        info.state = 3
    end 
    playList[info.file] = nil       
end

local function pauseInfo(info)
    if info.id then
        ccexp.AudioEngine:pause(info.id)
        info.state = 2
    end     
end

local function resumeInfo(info)
    if info.id and info.state==2 then
        ccexp.AudioEngine:resume(info.id)
        info.state = 1
    end  
end

function audio.playMusic(key, callback)
    local info = getSoundInfo(key)
    if info then
        audio.stopMusic()
        info.callback = callback
        if audio.bMusicEnable then
            playInfo(info)
        else
            info.state = 0
        end
        audio.curMusic = info
    end
end

function audio.pauseMusic()
    if audio.curMusic then
        pauseInfo(audio.curMusic)
    end
end

function audio.resumeMusic()
    if audio.bMusicEnable and audio.curMusic then
        resumeInfo(audio.curMusic)
    end
end


function audio.stopMusic()
    if audio.curMusic then
        stopInfo(audio.curMusic)
        audio.curMusic = nil 
    end
end

function audio.enableMusic(enable)
    if enable == audio.bMusicEnable then return end
    if enable then
        audio.bMusicEnable = true 
        if audio.curMusic then
            if audio.curMusic.state==2 then
                resumeInfo(audio.curMusic)
            elseif audio.curMusic.state==0 then
                playInfo(audio.curMusic)
            end
        end
    else
        audio.bMusicEnable = false
        if audio.curMusic then
            pauseInfo(audio.curMusic)
        end
    end
end

function audio.isMusicEnable()
    return audio.bMusicEnable
end

function audio.isEffectEnable()
    return audio.bEffectEnable
end

function audio.enableEffect(enable)
    if enable == audio.bEffectEnable then return end
    if enable then
        audio.bEffectEnable = true 
    else
        audio.bEffectEnable = false
    end
end

function audio.playEffect(key, callback)
    local info = getSoundInfo(key)
    if info then
        info.callback = callback
        if audio.bEffectEnable then
            playInfo(info)
--        else
--            info.state = 0
        end
    end
end

--function audio.stopEffect(id)
--    ccexp.AudioEngine:stop(id)
--end

function audio.playUI( view, nodename )
    local viewname = view:getName()
    local key = view.key
    --print(viewname, nodename)
    local view = soundInfo.UIBind[viewname]
    local sound = nil
    if view then
        local node = view[nodename]
        if node then
            sound = node
        elseif view.default then
            sound = view.default
        end
    end
    if not sound then
        sound = soundInfo.UIBind.default
    end

    if sound and sound~=0 then
        audio.playEffect(sound)
    end
end

return audio