cc.exports.profiler = {}

local Counters = {} 
local Names = {}
local Clocks = {}
local Times = {}
local function trace(event, line)
    local f = debug.getinfo(2, "f").func
    if event=='return' then
        local t = Clocks[f] and (os.clock() - Clocks[f]) or 0
        Times[f] = (Times[f] or 0) + t
        return
    end
    Clocks[f] = os.clock()
    if Counters[f] == nil then 
        -- first time `f' is called?   
        Counters[f] = 1   
        Names[f] = debug.getinfo(2, "Sn")  
    else 
        -- only increment the counter   
        Counters[f] = Counters[f] + 1  
    end
end

function profiler.start(name)
    Counters = {}
    Names = {}
    Clocks = {}
    Times = {}
    debug.sethook(trace, "cr")
end

function profiler.stop()
    debug.sethook()
    profiler.dump()
end

local function getname (func)
    local n = Names[func]
    if n.what == "C" then
        return n.name  
    end
    local loc = string.format("[%s]:%s <%f>",    n.short_src, n.linedefined, Times[func] or 0)
    if n.namewhat ~= "" then
        return string.format("%s (%s)", loc, n.name)
    else
        return string.format("%s", loc)
    end
end

function profiler.dump()
    print('<<<<====================== profiler =====================')
    local list = {}
    for func, t in pairs(Times) do
        if t and t>0 then
            list[#list+1] = { func, t}
        end
    end
    table.sort(list, function(a,b) return a[2]>b[2] end )
    for i=1, math.min(10 ,#list) do
        local func = list[i][1]
        print(getname(func), Counters[func]) 
    end 
    print('    ====================== profiler =====================>>>')
end

return profiler