cc.exports.smart = {}

smart.pool = {}

local Pool = {}
function Pool:new(name, viewname)
    local o = {}
	setmetatable(o, self)
	self.__index = self
    self.name = name
    self.viewname = viewname
    self.deactive = {}
    self.active = {}
    self.idx = 1
	return o
end

function Pool:createView()
    local n = #self.active
    local dn = #self.deactive
    if dn >0 then
        local v = self.deactive[dn]
        self.deactive[dn] = nil
        self.active[n+1] = v
        self:scheduleGC()
        return v
    end
    
	local obj = util.getApp():createView(self.name, self.viewname)
    obj:retain() 
    obj.ui.animation:retain()
    self.active[n+1] = obj
    self:scheduleGC()
    return obj
end

function Pool:scheduleGC()
    local curScene = util.getScene()
    local _self = self
    local function cb()
        _self:GC()
        self._scene = nil
        if #self.active>0 then
            _self:scheduleGC()
        end
    end
    if self._scene~= curScene then
        self._scene = curScene
        performWithDelay(self._scene, cb, 1)
    end
end

function Pool:GC()
    local n = #self.active
    local dn = #self.deactive
    for i=n,1,-1 do
        local v = self.active[i]
        if v:getReferenceCount()==1 then
            dn=dn+1
            self.deactive[dn] = v
            v.isAnimationRunning_ = false
            table.remove(self.active, i)
        end
    end
end

function smart.getPool(name, viewname)
    local v = smart.pool[name]
    local vn = viewname or '_nil'
    if v then
        if v[vn] then
            return v[vn]
        else
            local pool = Pool:new(name, viewname)
            v[vn] = pool
            return pool
        end 
    else
        v = {}
        local pool = Pool:new(name, viewname)
        v[vn] = pool
        smart.pool[name] = v
        return pool
    end
end

function smart.createView(name, viewname)
    local pool = smart.getPool(name, viewname)
    if pool then
        return pool:createView()
    end
    
end

function smart.GC(name, viewname)
    local pool = smart.getPool(name, viewname)
    if pool then
        return pool:GC()
    end
end

return smart