cc.exports.util = util or {}
require('json')

local _dialoglist = {}
function util.getUUID()
	local uuid = cc.UserDefault:getInstance():getStringForKey('USER_UUID')
	if uuid==nil or uuid=="" then
		uuid = GamePlayer:GetUserUUID()
		cc.UserDefault:getInstance():setStringForKey('USER_UUID', uuid)
	end
    return uuid
end

function util.getApp()
	return util.app
end

function util.getTableSize(t)
    local count = 0
    for _, v in pairs(t) do
        count = count + 1
    end
    return count
end

function util.getScene()
	return util.app:getScene()
end

function util.enterScene(sceneName, transition, time, more)
    return util.app:enterScene(sceneName, transition, time, more)
end

function util.createView(name, viewname)
    return util.app:createView(name, viewname)
end

function util.isShowDialog(name)
    return _dialoglist[name]~=nil
end
function util.showDialog(target, name, viewname)
    local node = util.createView(name, viewname)
    if not tolua.isnull(node) then
        if target then
            local tar = util.getScene().ui[target]
            if not tolua.isnull(tar) then
                _dialoglist[name] = node
                tar:add(node)
            else
                printInfo('not find target ' .. target)
            end
        else
            _dialoglist[name] = node
            local scene = util.getScene()
            if scene.ui and scene.ui.root then
                scene.ui.root:add(node)
            else
                scene:addChild(node)
            end
        end
    end
    return node
end

function util.closeDialog(name, cb)
    local node = _dialoglist[name]
    if node then
        util.playClose(node, cb)
        --node:removeFromParent()
        _dialoglist[name] = null
    end
end

function util.print(msg, ...)
	print(msg)
end

function util.delay(sec, callback)
    local scheduler = cc.Director:getInstance():getScheduler()
    local target = nil
	target = scheduler:scheduleScriptFunc(function(...) 
        scheduler:unscheduleScriptEntry(target)
        callback(...)
    end , sec, false)
    return target
end

function util.schedule(sec, callback)
    local scheduler = cc.Director:getInstance():getScheduler()
    return scheduler:scheduleScriptFunc(callback, sec, false)
    --
end

function util.unschedule(target)
    local scheduler = cc.Director:getInstance():getScheduler()
    scheduler:unscheduleScriptEntry(target)
end

function util.delayNode(sec, callback, node)
    node = node or util.getScene()
    return performWithDelay(node, callback, sec)
end

function util.scheduleNode(sec, callback, node)
    node = node or util.getScene()
    return schedule(node, callback, sec)
end

-- for delayNode or scheduleNode
function util.unscheduleNode(seq, node)
    node = node or util.getScene()
    node:stopAction(seq)
end

function util.loadJsonFile(file)
    return json.decode(cc.FileUtils:getInstance():getStringFromFile(file))
end
function util.loadSortJsonFile(file)
    return json.decode_order(cc.FileUtils:getInstance():getStringFromFile(file))
end

function util.tableCopy(ori_tab)
    if type(ori_tab) ~= "table" then
        return nil
    end
    
    local new_tab = {}
    for i, v in pairs(ori_tab) do
        local vtype = type(v)
        if vtype == "table" then
            new_tab[i] = util.tableCopy(v)
        else
            new_tab[i] = v
        end
    end
    return new_tab
end

function util.tableClear(ori_table)
    local result={}
    if type(ori_table)=="table" then
        local count = #ori_table
        for i=1,count do
            if ori_table[i]~="" then
                table.insert(result,ori_table[i])
            end
        end
    end
    return result
end

function util.playClose(view, cb)
    if not view.ui then return end

    if not tolua.isnull(view.ui.animation) and view.ui.animation:IsAnimationInfoExists('close') then
	    view:play('close')
        local info = view.ui.animation:getAnimationInfo('close')
        performWithDelay(view, function() view:close() if cb then cb() end end, (info.endIndex - info.startIndex)/60)
        if view.__back then
            util.playClose(view.__back)
            return
        end
    else
        if view.__back then
            util.playClose(view.__back, function()
                view:close()
                if cb then cb() end
            end)
            return
        end
        view:close()
        if cb then cb() end
    end
end

function util.isTimeBetween(time, from, to)
    if to>from then
        return from<=time and time<to
    else
        return time>=from or time<to
    end
    return false
end

-- 获得文本自适应高度
function util.getLabelSize(uiTxt)
    local l = cc.Label:create()
    l:setTTFConfig(uiTxt:getVirtualRenderer():getTTFConfig())
    local size = uiTxt:getContentSize()
    l:setContentSize(size.width, 0)
    l:setMaxLineWidth(size.width)
    l:setString(uiTxt:getString())
    return l:getContentSize()
end

-- 键盘按键处理 (内部调用)
function util.doKeyPressed()
    local function onKeyPressed(keyCode, ev)
        if keyCode == cc.KeyCode.KEY_BACK then
            print("BACK clicked!")
        elseif keyCode == cc.KeyCode.KEY_MENU  then
            print("MENU clicked!")
        elseif keyCode == cc.KeyCode.KEY_F1 then
            lang.prev()
        elseif keyCode == cc.KeyCode.KEY_F2 then
            lang.next()
        end
        event.dispatch('onKeyPressed', keyCode)
    end
    local function onKeyReleased(keyCode, ev)
        event.dispatch('onKeyReleased', keyCode)
    end
    local listener = cc.EventListenerKeyboard:create()
    listener:registerScriptHandler(onKeyPressed, cc.Handler.EVENT_KEYBOARD_PRESSED )
    listener:registerScriptHandler(onKeyReleased, cc.Handler.EVENT_KEYBOARD_RELEASED )

    local eventDispatcher = util.getScene():getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener:clone(), util.getScene())

end

--判断是否安卓平台
function util.isAndroid()
    return (cc.Application:getInstance():getTargetPlatform()==cc.PLATFORM_OS_ANDROID)
end

--判断是否IOS平台
function util.isIOS()
    local curPlatform=cc.Application:getInstance():getTargetPlatform()
    return (curPlatform==cc.PLATFORM_OS_IPHONE or curPlatform==cc.PLATFORM_OS_IPAD)
end

--判断是否windows平台
function util.isWindows()
    return (cc.Application:getInstance():getTargetPlatform()==cc.PLATFORM_OS_WINDOWS)
end

--判断是否Mac平台
function util.isMac()
    return (cc.Application:getInstance():getTargetPlatform()==cc.PLATFORM_OS_MAC)
end

function util.getUTF(str)
    local a,b,c = string.byte(str, 1, 3)
    if a==239 and b==187 and c==191 then
        str = string.sub(str, 4)
    end
    return str
end

-- 通过sprite创建ProgressTimer

function util.createProgressTimer(sprite, isBar)
    local parent = sprite:getParent()
    sprite:removeFromParent()
    local pt = cc.ProgressTimer:create(sprite)
    parent:addChild(pt)
    pt:setContentSize(sprite:getContentSize())
    pt:setAnchorPoint(0.5, 0.5)
    pt:setType(isBar and cc.PROGRESS_TIMER_TYPE_BAR or cc.PROGRESS_TIMER_TYPE_RADIAL)
    pt:setPosition(sprite:getPosition())
    return pt
end

function util.HttpManagerPOST(ip,cmd,cbFun)
    local HttpManager = {}
    HttpManager.SERVER_IP = ip
    HttpManager.http=nil
    function HttpManager.sendMessage(fun)
        local conect = cc.XMLHttpRequest:new()
        conect.responseType = cc.XMLHTTPREQUEST_RESPONSE_STRING
        conect:open("POST",HttpManager.SERVER_IP)
        conect:registerScriptHandler(fun)
        conect:send(cmd)
        HttpManager.http = conect
    end

    function HttpManager.managerFun()
        local josnString = HttpManager.http.response
        print("josnString···",josnString)
        local output = nil;
        if josnString ~= "" then
          output = json.decode(josnString,1)
        end
        if output ~= nil then
            cbFun(output)     
        end
    end
    HttpManager.sendMessage(HttpManager.managerFun)
end

function util.SendLuaFuncID2Java(func)
    local luaj = require "cocos.cocos2d.luaj"
	local classname = "org/cocos2dx/lua/AppActivity"
	local args = {func}
	local sigs = "(I)V"

	local ok,ret = luaj.callStaticMethod(classname, "GetLuaFuncID", args, sigs)
end

return util