
local LayoutBase = class("LayoutBase", ccui.Layout)

function LayoutBase:ctor(app, name, viewname)
    self:enableNodeEvents()
    self.app_ = app
    self.name_ = name

    -- check CSB resource file
    local res = viewname or rawget(self.class, "RESOURCE_FILENAME")
    if res then
        self:createResourceNode(res)
    end

    local binding = rawget(self.class, "RESOURCE_BINDING")
    if res and binding then
        self:createResourceBinding(binding)
    end

    if self.onCreate then self:onCreate() end
end

function LayoutBase:getApp()
    return self.app_
end

function LayoutBase:getScene()
    return self.app_:getScene()
end

function LayoutBase:getName()
    return self.name_
end

function LayoutBase:getResourceNode()
    return self.resourceNode_
end

function LayoutBase:getResourceUI()
    return self.resourceUI_
end

function LayoutBase:createResourceNode(resourceFilename)
    if self.resourceNode_ then
        self.resourceNode_:removeSelf()
        self.resourceNode_ = nil
        self.resourceUI_ = nil
    end

    local cb = function ( path,node,funcName )
        local function btnCallback(  node,eventType  )
            local func = self[funcName]
            local ret = nil
            if func then
                if not eventType or eventType==0 then
                    audio.playUI(self, funcName)
                end
                if node.UserData==nil then
                    ret = func(self, node, eventType)
                else
                    ret = func(self, node, eventType, node.UserData[1])
                end
            else
                print(path..' Event '..funcName..' not find!')                    
            end
            if funcName=='onClose' and (not ret) then     -- Auto Close
                util.playClose(self)
            end
        end
        node.__callback = btnCallback
        return btnCallback
    end
--    local frameSize = cc.Director:getInstance():getOpenGLView():getFrameSize()

--    local status 
    local view 
--    if frameSize.width/frameSize.height <= 1.774 then
--        status, view = xpcall(function()
--                return require(resourceFilename.."_4_3")
--            end, function(msg) return require(resourceFilename) end
--        )

--    else
        view = require(resourceFilename)
--    end
    self.resourceUI_ = view.create(cb) -- cc.CSLoader:createNode(resourceFilename)
    self.ui = self.resourceUI_
    self.resourceNode_ = self.resourceUI_.root 
    self.resourceNode_:setCascadeColorEnabled(true)
    self.resourceNode_:setCascadeOpacityEnabled(true)
    self:setCascadeColorEnabled(true)
    self:setCascadeOpacityEnabled(true)
    self:setContentSize(self.resourceNode_:getContentSize())
    local layout = ccui.LayoutComponent:bindLayoutComponent(self.resourceNode_)
    layout:setPercentWidthEnabled(true)
    layout:setPercentHeightEnabled(true)
    layout:setPercentWidth(1.0000)
    layout:setPercentHeight(1.0000)
    assert(self.resourceNode_, string.format("LayoutBase:createResourceNode() - load resouce node from file \"%s\" failed", resourceFilename))
    self:addChild(self.resourceNode_)

    -- Auto Show
    if self.ui.animation:IsAnimationInfoExists('show') then
        self:play('show')
    elseif self.ui.animation:IsAnimationInfoExists('loop') then
        self:play('loop', true)
    end
end

function LayoutBase:createResourceBinding(binding)
    assert(self.resourceNode_, "LayoutBase:createResourceBinding() - not load resource node")
    for nodeName, nodeBinding in pairs(binding) do
        local node = self.resourceNode_:getChildByName(nodeName)
        if nodeBinding.varname then
            self[nodeBinding.varname] = node
        end
        for _, event in ipairs(nodeBinding.events or {}) do
            if event.event == "touch" then
                node:onTouch(handler(self, self[event.method]))
            end
        end
    end
end

function LayoutBase:showWithScene(transition, time, more)
    self:setVisible(true)
    local scene = display.newScene(self.name_)
    scene:addChild(self)
    display.runScene(scene, transition, time, more)
    return self
end

function LayoutBase:showView(name, closeCB)
    local view = self:getApp():createView(name)
    self:addChild(view)
    view._closeCB = closeCB
    return view
end

function LayoutBase:close()
    if self._closeCB then
        self._closeCB()
    end
    self:removeFromParent()
end

function LayoutBase:addEvent(eventname)
    if dispatchname then
        local handle = nil
        local function listenerDispatchCallback(ev)
            if tolua.isnull(self) then
                local dispatch = event.get(dispatchname)
                dispatch:removeEventListener(handle)
            else
                self[eventname](self, ev, ev._usedata)
            end
        end        
        local dispatch = event.get(dispatchname)
        handle = dispatch:addEventListener(eventname, listenerDispatchCallback)
        return handle
    end
    local function listenerCallback(ev)
        self[eventname](self, ev, ev._usedata)
    end
    return event.addNodeEvent(eventname, listenerCallback, self)
end

function LayoutBase:runAnimation()
    local ui = self:getResourceUI()
    if not ui then return end
    if not self.isAnimationRunning_ then
        ui.root:runAction(ui.animation)
        self.isAnimationRunning_ = true
    end
end

function LayoutBase:play(animation, loop)
    local ui = self:getResourceUI()
    if not ui then return end
    self:runAnimation()
    local function frameEvent(frame)
        local funcName = frame:getEvent()
        if funcName=="" then
            return
        end
        local func = self[funcName]
        if func then
            func(self, frame:getNode(), frame)
        else
            error('Frame Event '..funcName..' not find!')
        end
        if funcName=='onEnd' then
            self:close()
        end
    end
    ui.animation:setFrameEventCallFunc(frameEvent)
    ui.animation:play(animation, loop)
end

return LayoutBase
