--[[
Serialzation bytes stream like ActionScript flash.utils.ByteArray.
It depends on lpack.
A sample: https://github.com/zrong/lua#ByteArray

@see http://underpop.free.fr/l/lua/lpack/
@see http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/utils/ByteArray.html
@author zrong(zengrong.net)

Creation 2013-11-14
Last Modification 2014-12-25
]]
local ByteArray = class("ByteArray2")

ByteArray.ENDIAN_LITTLE = "ENDIAN_LITTLE"
ByteArray.ENDIAN_BIG = "ENDIAN_BIG"
ByteArray.radix = {[10]="%03u",[8]="%03o",[16]="%02X"}

function ByteArray:ctor()
	self._buf = nil
	self._pos = 1
end

function ByteArray:getLen()
	return string.len(self._buf)
end

--function ByteArray:getAvailable()
--	return #self._buf - self._pos + 1
--end

function ByteArray:getPos()
	return self._pos
end

function ByteArray:setPos(__pos)
	self._pos = __pos
	return self
end

--- Get all byte array as a lua string.
-- Do not update position.
function ByteArray:getBytes(__offset, __length)
	__offset = __offset or 1
	--__length = __length or string.len(self._buf)
	return string.sub(self._buf, __offset, __length)
end

function ByteArray:readBool()
	-- When char > 256, the readByte method will show an error.
	-- So, we have to use readChar
	--return self:readChar() ~= 0
    return self:readByte() ~= 0
end

function ByteArray:writeBool(__bool)
	if __bool then 
		self:writeByte(1)
	else
		self:writeByte(0)
	end
	return self
end

--function ByteArray:readFloat()
--	local __, __v = string.unpack(self:readBuf(4), self:_getLC("f"))
--	return __v
--end

--function ByteArray:writeFloat(__float)
--	local __s = string.pack( self:_getLC("f"),  __float)
--	self:writeBuf(__s)
--	return self
--end

function ByteArray:readInt()
	local __, __v = string.unpack(string.sub(self._buf, self._pos, self._pos+3), "=i")
    self._pos=self._pos+4
	return __v
end

--function ByteArray:writeInt(__int)

--	return self
--end

function ByteArray:readUInt()
	local __, __v = string.unpack(string.sub(self._buf, self._pos, self._pos+3), "=I")
    self._pos=self._pos+4
	return __v
end

--function ByteArray:writeUInt(__uint)

--	return self
--end

function ByteArray:readShort()
	local __, __v = string.unpack(string.sub(self._buf, self._pos, self._pos+1), "=h")
    self._pos=self._pos+2
	return __v
end

--function ByteArray:writeShort(__short)

--	return self
--end

function ByteArray:readUShort()
	local __, __v = string.unpack(string.sub(self._buf, self._pos, self._pos+1), "=H")
    self._pos=self._pos+2
	return __v
end

function ByteArray:writeUShort(__ushort)
--	local __s = string.pack("=H",  __ushort)
--	self:writeBuf(__s)
	return self
end

--function ByteArray:readUByte()

--end

--function ByteArray:writeUByte(__ubyte)

--	return self
--end

function ByteArray:readString(__len)
	if __len == 0 then return "" end
    local ret = string.sub(self._buf, self._pos, self._pos+__len-1)
    self._pos=self._pos+__len	
	return ret
end

--function ByteArray:writeString(__string)

--	return self
--end

----- Read some bytes from buf
---- @return a bit string
--function ByteArray:readBytes(__bytes, __offset, __length)

--end

----- Write some bytes into buf
--function ByteArray:writeBytes(__bytes, __offset, __length)

--end

--- Actionscript3 readByte == lpack readChar
-- A signed char
function ByteArray:readChar()
    local ret = string.sub(self._buf, self._pos, self._pos)
    self._pos=self._pos+1
    return ret
end

----- Use the string.pack of lpack api to write a byte.
--function ByteArray:writeChar(__char)

--	return self
--end

--- Use the lua string library to get a byte
-- A unsigned char
function ByteArray:readByte()
    local ret = string.byte(self._buf, self._pos)
    self._pos=self._pos+1
    return ret
end

----- Use the string.char of lua standard library to write a byte.
---- The byte is a number between 0 and 255, otherwise, Lua will throw an error.
--function ByteArray:writeByte(__byte)

--	return self
--end

----- Read a byte array as string from current position, then update the position.
--function ByteArray:readBuf(__len)
--	--printf("readBuf,len:%u, pos:%u", __len, self._pos)
--	local __ba = self:getBytes(self._pos, self._pos + __len - 1)
--	self._pos = self._pos + __len
--	return __ba
--end

--- Write a encoded char array into buf
function ByteArray:writeBuf(__s)
	self._buf = __s
	return self
end

----------------------------------------
-- private
----------------------------------------
--function ByteArray:_checkAvailable()
--	assert(#self._buf >= self._pos, string.format("End of file was encountered. pos: %d, len: %d.", self._pos, #self._buf))
--end



return ByteArray
