local BA = import(".ByteArray")
local BA2 = import(".ByteArray2")
local tagNetCmd = import(".tagNetCmd")
local netIF = class("netInterface")
local network = class("network")

lzo.init()

local function readShort(v)
    local __, __v = string.unpack(v, "=h")
    return __v
end

local function printBytes(v)
    local b = BA:create()
    b:writeBuf(v)
    b:setPos(1)
    local str = ''
    for i=1, string.len(v) do
        str = str .. b:readByte()..' '
    end
    print(str)
end

function netIF:GetPack()
    if self.waitFrame then
        self.waitFrame=self.waitFrame-1 
        if self.waitFrame<0 then
            self.waitFrame = nil
        end
        return
    end
    while(true) do
        local str_len = string.len(self.pack_data)
        if str_len<2 then return end
        local pack_len = readShort(self.pack_data)
        if str_len<pack_len then return end
        local data_len = readShort(string.sub(self.pack_data, 3, 4))
        local data = string.sub(self.pack_data, 5, pack_len)
        if data_len>128 then
            local d, l = lzo.decompress(data, pack_len-4)
            if l==data_len then
                data = d
            else
                data = nil
            end
        end
        
        if str_len== pack_len then
            self.pack_data = ""
        else
            self.pack_data = string.sub(self.pack_data, pack_len+1)
        end

        if data then
            self:recv(data)
        end

        if self.waitFrame then
            return
        end
    end
end


function netIF:GetClientPack(client)
    while(true) do
        local pack_data = self.clientData[client]
        local str_len = string.len(pack_data)
        if str_len<2 then return end
        local pack_len = readShort(pack_data)
        if str_len<pack_len then return end
        local data = string.sub(pack_data, 3, pack_len)
        
        if str_len== pack_len then
            self.clientData[client] = ""
        else
            self.clientData[client] = string.sub(pack_data, pack_len+1)
        end

        if data then
            self:recv(data, client)
        end
    end
end

function netIF:listen(port)
    local addrifo = socket.dns.getaddrinfo("www.baidu.com")
    local isipv6_only = false
    if addrifo ~= nil then  
        for k,v in pairs(addrifo) do  
            if v.family == "inet6" then  
                isipv6_only = true  
                break  
            end  
        end  
    end  
    if isipv6_only then  
        self.tcp = socket.tcp6()  
    else  
        self.tcp = socket.tcp()  
    end
    self:unSchedule()
    self.client = {}
    self.clientData = {}
    self.scheduleID = util.schedule(0, function()
        local sl = socket.select({ self.tcp }, {},  0)
        if #sl>0 then
            local client = self.tcp:accept()
            self.client[#self.client+1] = client
            if self.onAccept then
                self.onAccept()
            end
        end

        local sl, sr = socket.select(self.client, self.client,  0)
        if #sr>0 then
--            if not self.isConnect then
--                self.isConnect = true
--                if self.onConnectFunc then
--                    self.onConnectFunc()
--                end
--            end
--        else
--            if self.isConnect then
--                self:close()
--                return 
--            end
        end
        if #sl>0 then
            for i=1, #sl do
                local client = sl[i]
                client:settimeout(0)
                local data = self.clientData[client] or ""
                while(true) do
                    local v, err, r = client:receive(1)
                    if err=='closed' then client:close() return end
                    if err then
                        if string.len(r)==0 then break end
                        data = data .. r
                    else
                        data = data .. v
                    end
                end
                self.clientData[client] = data
                self:GetClientPack(client)
            end
            
        end

    end)
    self.tcp:settimeout(0)
    
    self.isConnect = false
    self.isServer = true
    local myip = socket.dns.toip(socket.dns.gethostname())
    self.tcp:bind(myip, port)
    self.tcp:listen(0)
end

function netIF:getTcp(adr)
    local tcp = nil
    local addrifo = nil
    local arrAdr = string.split(adr, ".")
    if arrAdr[1] and (arrAdr[1] == "10" or arrAdr[1] == "172" or arrAdr[1] == "192") then
      
        addrifo = nil
    else
        
        addrifo = socket.dns.getaddrinfo("www.baidu.com")
    end

    local isipv6_only = false
    if addrifo ~= nil then
        for k,v in pairs(addrifo) do
            if v.family == "inet6" then
                isipv6_only = true  
                break
            end  
        end  
    end
    if isipv6_only then  
        tcp = socket.tcp6()  
    else  
        tcp = socket.tcp()  
    end

    return tcp
end

function netIF:connect(adr, port)
    self.tcp = self:getTcp(adr)     --|ipv6
    self.pack_data = ""
    self:unSchedule()
    self.leftLen = 1
    self.scheduleID = util.schedule(0, function()
        local sl, sr = socket.select({ self.tcp }, {self.tcp},  0)
        if #sr>0 then
            if not self.isConnect then
                self.isConnect = true
                if self.onConnectFunc then
                    self.onConnectFunc()
                end
            end
        else
            if self.isConnect then
                self:close()
                return 
            end
        end
        if #sl>0 then
        	--tcp:settimeout(1)
        end
            while(true) do
                local v, err, r = self.tcp:receive(self.leftLen)
                --tcp:settimeout(0)
                    
                if err~=nil and err~='timeout' then
                    self:close()
                    return
                end
                if err then
                    if string.len(r)==0 then break end
                    self.pack_data = self.pack_data .. r
                else
                    self.pack_data = self.pack_data .. v
                end

                local str_len = string.len(self.pack_data)
                if str_len>2 then 
                    local pack_len = readShort(self.pack_data)
                    if str_len>=pack_len then
                        self:GetPack()
                        self.leftLen = 1
                    else
                        self.leftLen = pack_len-str_len
                    end
                end
            end
            self:GetPack()
        --end

    end)
    self.tcp:settimeout(0)
    
    self.isConnect = false
    print("==============netInterface=== tcp:connect",adr,port)
    self.tcp:connect(adr, port)

end


function netIF:getPingTime(adr, port, timeOut)
    local tcp = self:getTcp(adr)
    tcp:settimeout(timeOut)
    local startTime = os.clock()
    tcp:connect(adr, port)
    local endTime = os.clock()
    tcp:close()
    return endTime - startTime
end

function netIF:close()
    if self.isConnect then
        self.isConnect = false
        if self.onClose then
            self.onClose()
        end
    end
    self.isConnect = false
    if self.tcp then
        self:unSchedule()

        self.tcp:close()
    end
end

function netIF:unSchedule()
    if self.scheduleID then
        util.unschedule(self.scheduleID)
        self.scheduleID = nil
    end
end


function netIF:isConnected()
    return self.isConnect or false
end

function netIF:send(msg)
    if CC_SHOW_NET_MSG and not self.isServer then
        if not (tagNetCmd.igoreList and tagNetCmd.igoreList[msg.dwID]) then
            print('[Send] : '..msg.__cname..'['..(msg.dwSize+10)..'B]')
        end
    end
    local array = BA:create()
    local size = msg:getSize()
    array:writeShort(size+10 + 2)
    array:writeUInt(msg.dwID)
    array:writeShort(size+10)
    array:writeInt(msg.dwParam)
    msg:write(array, msg)
    local p = array:getBytes()
    self.tcp:send(p)
end

function netIF:sendTo(client, msg)
    if CC_SHOW_NET_MSG and not self.isServer then
        if not (tagNetCmd.igoreList and tagNetCmd.igoreList[msg.dwID]) then
            print('[Send] : '..msg.__cname..'['..(msg.dwSize+10)..'B]')
        end
    end
    local array = BA:create()
    local size = msg:getSize()
    

    if size +10 + 2 > 128 then
        local ar = BA:create()
        ar:writeUInt(msg.dwID)
        ar:writeShort(size+10)
        ar:writeInt(msg.dwParam)
        msg:write(ar, msg)
        local p = ar:getBytes()
        local d, l = lzo.compress(p, size +10 + 2)
        array:writeShort(l + 4)
        array:writeShort(size+10 + 2)
        array:writeBuf(d)
    else
        array:writeShort(size+10 + 2 + 2)
        array:writeShort(size+10 + 2)
        array:writeUInt(msg.dwID)
        array:writeShort(size+10)
        array:writeInt(msg.dwParam)
        msg:write(array, msg)
    end
    
    local p = array:getBytes()
        
    client:send(p)
end

function netIF:recv(data, client)
    local array = BA2:create()
    array:writeBuf(data)
    array:setPos(1)
    local dwID = array:readUInt()
    local dwSize = array:readShort()
    local dwParam = array:readInt()
    
    local tagClass = tagNetCmd.CMD_CLASS(dwID)
    
    if tagClass then
        if CC_SHOW_NET_MSG and not self.isServer then
            if not (tagNetCmd.igoreList and tagNetCmd.igoreList[dwID]) then
                print('[Recv] : '..tagClass.__cname..'['..dwSize..'B]')
            end
        end
        local tagObj = tagClass:read(array)
        if self.onRecv then
            self.onRecv(tagObj, client)
        end
    else
        print('===================================Not Reg CMD ID = '.. dwID.. '  Size = '.. dwSize)
    end
    
end

function netIF:on(eventName, func)
    if eventName=='recv' then
        self.onRecv = func
    elseif eventName=='connect' then
        self.onConnectFunc = func
    elseif eventName=='accept' then
        self.onAccept = func
    elseif eventName=='send' then
        self.onSend = func
    elseif eventName=='close' then
        self.onClose = func
    end
end

function netIF:wait(frame)
    self.waitFrame = frame
end

return netIF
